@@ -2,23 +2,26 @@ package containerruntimeconfig
 
 import (
 	"bytes"
+	"context"
 	"encoding/json"
 	"errors"
 	"fmt"
 	"reflect"
+	"strconv"
 
 	"github.com/BurntSushi/toml"
 	"github.com/containers/image/docker/reference"
 	"github.com/containers/image/pkg/sysregistriesv2"
 	signature "github.com/containers/image/signature"
 	storageconfig "github.com/containers/storage/pkg/config"
-	ign3types "github.com/coreos/ignition/v2/config/v3_1/types"
+	ign3types "github.com/coreos/ignition/v2/config/v3_2/types"
 	"github.com/golang/glog"
 	apicfgv1 "github.com/openshift/api/config/v1"
 	apioperatorsv1alpha1 "github.com/openshift/api/operator/v1alpha1"
 	"github.com/openshift/runtime-utils/pkg/registries"
 	"github.com/vincent-petithory/dataurl"
 	corev1 "k8s.io/api/core/v1"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 
 	mcfgv1 "github.com/openshift/machine-config-operator/pkg/apis/machineconfiguration.openshift.io/v1"
 	ctrlcommon "github.com/openshift/machine-config-operator/pkg/controller/common"
@@ -26,11 +29,12 @@ import (
 )
 
 const (
-	minLogSize           = 8192
-	minPidsLimit         = 20
-	storageConfigPath    = "/etc/containers/storage.conf"
-	registriesConfigPath = "/etc/containers/registries.conf"
-	policyConfigPath     = "/etc/containers/policy.json"
+	minLogSize              = 8192
+	minPidsLimit            = 20
+	storageConfigPath       = "/etc/containers/storage.conf"
+	registriesConfigPath    = "/etc/containers/registries.conf"
+	searchRegDropInFilePath = "/etc/containers/registries.conf.d/01-image-searchRegistries.conf"
+	policyConfigPath        = "/etc/containers/policy.json"
 	// CRIODropInFilePathLogLevel is the path at which changes to the crio config for log-level
 	// will be dropped in this is exported so that we can use it in the e2e-tests
 	CRIODropInFilePathLogLevel   = "/etc/crio/crio.conf.d/01-ctrcfg-logLevel"
@@ -71,7 +75,7 @@ type tomlConfigCRIOLogLevel struct {
 type tomlConfigCRIOPidsLimit struct {
 	Crio struct {
 		Runtime struct {
-			PidsLimit int64 `toml:"pids_limit,omitemtpy"`
+			PidsLimit int64 `toml:"pids_limit,omitempty"`
 		} `toml:"runtime"`
 	} `toml:"crio"`
 }
@@ -176,8 +180,55 @@ func getManagedKeyCtrCfgDeprecated(pool *mcfgv1.MachineConfigPool) string {
 	return fmt.Sprintf("99-%s-%s-containerruntime", pool.Name, pool.ObjectMeta.UID)
 }
 
-func getManagedKeyCtrCfg(pool *mcfgv1.MachineConfigPool, client mcfgclientset.Interface) (string, error) {
-	return ctrlcommon.GetManagedKey(pool, client, "99", "containerruntime", getManagedKeyCtrCfgDeprecated(pool))
+// nolint: dupl
+func getManagedKeyCtrCfg(pool *mcfgv1.MachineConfigPool, client mcfgclientset.Interface, cfg *mcfgv1.ContainerRuntimeConfig) (string, error) {
+	// Get all the ctrcfg CRs
+	ctrcfgList, err := client.MachineconfigurationV1().ContainerRuntimeConfigs().List(context.TODO(), metav1.ListOptions{})
+	if err != nil {
+		return "", fmt.Errorf("error listing container runtime configs: %v", err)
+	}
+	// If there is no ctrcfg in the list, return the default MC name with no suffix
+	if ctrcfgList == nil || ctrcfgList.Items == nil {
+		return ctrlcommon.GetManagedKey(pool, client, "99", "containerruntime", getManagedKeyCtrCfgDeprecated(pool))
+	}
+	for _, ctrcfg := range ctrcfgList.Items {
+		if ctrcfg.Name == cfg.Name {
+			val, ok := ctrcfg.GetAnnotations()[ctrlcommon.MCNameSuffixAnnotationKey]
+			// If we find a matching ctrcfg and it is the only one in the list, then return the default MC name with no suffix
+			if !ok && len(ctrcfgList.Items) < 2 {
+				return ctrlcommon.GetManagedKey(pool, client, "99", "containerruntime", getManagedKeyCtrCfgDeprecated(pool))
+			}
+			// Otherwise if an MC name suffix exists, append it to the default MC name and return that as this ctrcfg exists and
+			// we are probably doing an update action on it
+			if val != "" {
+				return fmt.Sprintf("99-%s-generated-containerruntime-%s", pool.Name, val), nil
+			}
+		}
+	}
+
+	// If we are here, this means that a new ctrcfg was created, so we have to calculate the suffix value for its MC name
+	suffixNum := 0
+	// Get the suffix value of the second to last item in the ctrcfg list. We use the second to last item because
+	// the most recent ctrcfg being created has already been added to the list and is the ctrcfg object that we are
+	// trying to figure out the suffix value for
+	val, ok := ctrcfgList.Items[len(ctrcfgList.Items)-2].GetAnnotations()[ctrlcommon.MCNameSuffixAnnotationKey]
+	if ok {
+		// Convert the suffix value to int so we can add 1 to it for the MC name for the latest ctrcfg object
+		suffixNum, err = strconv.Atoi(val)
+		if err != nil {
+			return "", fmt.Errorf("error converting %s to int: %v", val, err)
+		}
+	}
+	// The max suffix value that we can go till with this logic is 9 - this means that a user can create up to 10 different ctrcfg CRs.
+	// However, if there is a ctrcfg-1 mapping to mc-1 and ctrcfg-2 mapping to mc-2 and the user deletes ctrcfg-1, it will delete mc-1 but
+	// then if the user creates a ctrcfg-new it will map to mc-3. This is what we want as the latest ctrcfg created should be higher in priority
+	// so that those changes can be rolled out to the nodes. But users will have to be mindful of how many ctrcfg CRs they create. Don't think
+	// anyone should ever have the need to create 10 when they can simply update an existing ctrcfg unless it is to apply to another pool.
+	if suffixNum+1 > 9 {
+		return "", fmt.Errorf("max number of supported ctrcfgs (10) has been reached. Please delete old ctrcfgs before retrying")
+	}
+	// Return the default MC name with the suffixNum+1 value appended to it
+	return fmt.Sprintf("99-%s-generated-containerruntime-%s", pool.Name, strconv.Itoa(suffixNum+1)), nil
 }
 
 // Deprecated: use getManagedKeyReg
@@ -207,7 +258,7 @@ func wrapErrorWithCondition(err error, args ...interface{}) mcfgv1.ContainerRunt
 	if len(args) > 0 {
 		format, ok := args[0].(string)
 		if ok {
-			condition.Message = fmt.Sprintf(format, args[:1]...)
+			condition.Message = fmt.Sprintf(format, args[1:]...)
 		}
 	}
 	return *condition
@@ -244,7 +295,7 @@ func addTOMLgeneratedConfigFile(configFileList []generatedConfigFile, path strin
 	return configFileList, nil
 }
 
-// createCRIODropinFiles gets the data from the CRD and creates the respective drio in file in /etc/crio/crio.conf.d
+// createCRIODropinFiles gets the data from the CRD and creates the respective drop in file in /etc/crio/crio.conf.d
 // We create different drop-in files for each CRI-O field that can be changed by the ctrcfg CR
 // this ensures that we don't have to rely on hard coded defaults that might cause problems
 // in future if something in cri-o or the templates used by the MCO changes
@@ -281,6 +332,22 @@ func createCRIODropinFiles(cfg *mcfgv1.ContainerRuntimeConfig) []generatedConfig
 	return generatedConfigFileList
 }
 
+// updateSearchRegistriesConfig gets the ContainerRuntimeSearchRegistries data from the Image CRD
+// and creates a drop-in file for it at /etc/containers/registries.conf.d
+func updateSearchRegistriesConfig(searchRegs []string) []generatedConfigFile {
+	var (
+		generatedConfigFileList []generatedConfigFile
+		err                     error
+	)
+	tomlConf := sysregistriesv2.V2RegistriesConf{}
+	tomlConf.UnqualifiedSearchRegistries = searchRegs
+	generatedConfigFileList, err = addTOMLgeneratedConfigFile(generatedConfigFileList, searchRegDropInFilePath, tomlConf)
+	if err != nil {
+		glog.Warningln("error updating user changes for containerRuntimeSearchRegistries to registries.conf.d: ", err)
+	}
+	return generatedConfigFileList
+}
+
 func updateRegistriesConfig(data []byte, internalInsecure, internalBlocked []string, icspRules []*apioperatorsv1alpha1.ImageContentSourcePolicy) ([]byte, error) {
 	tomlConf := sysregistriesv2.V2RegistriesConf{}
 	if _, err := toml.Decode(string(data), &tomlConf); err != nil {
